import os
import time
import sqlite3
import threading
from sqlite3 import Error
from datetime import datetime, timedelta

def create_conn(db_file):
    """ create a database connection to a SQLite3 database """
    try:
        conn = sqlite3.connect(db_file, check_same_thread=False)
        conn.commit()
        return conn
    except Error as e:
        print(e)

    return None

def create_table(conn, create_table_sql):
    try:
        c = conn.cursor()
        c.execute(create_table_sql)
        conn.commit()
        return conn
    except Error as e:
        print (e)

    return None


### user table ###
def create_user(conn, user):
    """ Create a new light row into the lights table """
    try:
        sql = """ INSERT INTO users(user, uid) VALUES(?,?) """

        c = conn.cursor()
        c.execute(sql, user)
        conn.commit()
        return int(c.lastrowid)
    except Error as e:
        print (e)

    return None
def check_user_id(conn, uid):
    try:
        c = conn.cursor()
        c.execute(" SELECT * FROM users WHERE rfid_uid=?",(uid,))
        user = c.fetchone()
    except Error as e:
        print (e)
    return user
def delete_light(conn, id):
    """ Delete a light by light id """
    try:
        sql = """ DELETE FROM lights where rowid=? """
        print("Deleting light {}").format(id)
        cur = conn.cursor()
        cur.execute(sql, (id,))
        conn.commit()
    except Error as e:
        print (e)

    return None

def delete_all_lights(conn):
    """ Delete all rows in the lights table """
    try:
        sql = """ DELETE FROM lights """
        cur = conn.cursor()
        cur.execute(sql)
        conn.commit()
    except Error as e:
        print (e)

    return None
def get_data_homepage(conn):
    line=[]
    try:
        c = conn.cursor()
        c.execute(" SELECT * FROM temperature WHERE room = 1 ORDER BY rowid DESC")
        temperature1 = c.fetchone()
        if temperature1:
            room,temperature,humidity,entry = temperature1
            line.append(temperature)
            line.append(humidity)
        else:
            line.append(None)
            line.append(None)
        c.execute(" SELECT * FROM temperature WHERE room = 2 ORDER BY rowid DESC")
        temperature2 = c.fetchone()
        if temperature2:
            room2,temperature2,humidity2,entry2 = temperature2
            line.append(temperature2)
            line.append(humidity2)
        else:
            line.append(None)
            line.append(None)
        return line
    except Error as e:
        print (e)

def select_all_accessdata(conn):
    """ Query last 10 rows in the temp table """
    user_list = []
    try:
        c = conn.cursor()
        c.execute(" SELECT * FROM rfid ORDER BY rowid DESC")
        users = c.fetchall()
        if users:
            for i in users:
                    line=[]
                    room,uid,entry,user = i
                    datetime_value = datetime.strptime(entry, '%Y-%m-%d %H:%M:%S.%f')
                    line.append("{:%d/%m/%y %H:%M:%S}".format(datetime_value))
                    line.append(room)
                    line.append(user)
                    user_list.append(line)
                    user_list_reversed = user_list[::-1]
            return user_list_reversed
        else:
            print("empty")
            return None
    except Error as e:
        print (e)

def select_all_room1_temperature(conn):
    """ Query last 10 rows in the temp table """
    temperature_list = []
    try:
        c = conn.cursor()
        c.execute(""" SELECT * FROM temperature WHERE room = 1 ORDER BY rowid DESC LIMIT 10""")
        print("Fetching last 10 rows from lights table")
        lights = c.fetchall()
        if lights:
            for i in lights:
                    line=[]
                    room,temperature,humidity,entry = i
                    datetime_value = datetime.strptime(entry, '%Y-%m-%d %H:%M:%S.%f')
                    line.append("{:%d/%m/%y %H:%M:%S}".format(datetime_value))
                    line.append(humidity)
                    temperature_list.append(line)
                    temperature_list_reversed = temperature_list[::-1]
            return temperature_list_reversed
        else:
            print("empty")
            return None
    except Error as e:
        print (e)

    return None
def select_all_room1_temperature_table(conn):
    """ Query  all rows in the temp table """
    temperature_list = []
    try:
        c = conn.cursor()
        c.execute(""" SELECT * FROM temperature WHERE room = 1 ORDER BY rowid""")
        print("Fetching last 10 rows from lights table")
        lights = c.fetchall()
        if lights:
            for i in lights:
                    line=[]
                    room,temperature,humidity,entry = i
                    datetime_value = datetime.strptime(entry, '%Y-%m-%d %H:%M:%S.%f')
                    line.append("{:%d/%m/%y %H:%M:%S}".format(datetime_value))
                    line.append(humidity)
                    line.append(temperature)
                    temperature_list.append(line)
                    temperature_list_reversed = temperature_list[::-1]
            return temperature_list_reversed
        else:
            print("empty")
            return None
    except Error as e:
        print (e)

    return None

def select_all_room2_temperature(conn):
    """ Query last 10 rows in the temp table """
    temperature_list = []
    try:
        c = conn.cursor()
        c.execute(""" SELECT * FROM temperature WHERE room = 2 ORDER BY rowid DESC LIMIT 10""")
        print("Fetching last 10 rows from lights table")
        lights = c.fetchall()
        if lights:
            for i in lights:
                    line=[]
                    room,temperature,humidity,entry = i
                    datetime_value = datetime.strptime(entry, '%Y-%m-%d %H:%M:%S.%f')
                    line.append("{:%d/%m/%y %H:%M:%S}".format(datetime_value))
                    line.append(humidity)
                    temperature_list.append(line)
                    temperature_list_reversed = temperature_list[::-1]
            return temperature_list_reversed
        else:
            print("empty")
            return None
    except Error as e:
        print (e)

    return None
def select_all_room2_temperature_table(conn):
    """ Query  all rows in the temp table """
    temperature_list = []
    try:
        c = conn.cursor()
        c.execute(""" SELECT * FROM temperature WHERE room = 2 ORDER BY rowid""")
        print("Fetching last 10 rows from lights table")
        lights = c.fetchall()
        if lights:
            for i in lights:
                    line=[]
                    room,temperature,humidity,entry = i
                    datetime_value = datetime.strptime(entry, '%Y-%m-%d %H:%M:%S.%f')
                    line.append("{:%d/%m/%y %H:%M:%S}".format(datetime_value))
                    line.append(humidity)
                    line.append(temperature)
                    temperature_list.append(line)
                    temperature_list_reversed = temperature_list[::-1]
            return temperature_list_reversed
        else:
            print("empty")
            return None
    except Error as e:
        print (e)

    return None


def select_all_room2_lights(conn):
    """ Query last 10 rows in the temp table """
    lights_list = []
    try:
        c = conn.cursor()
        c.execute(""" SELECT * FROM lights WHERE room = 2 ORDER BY rowid DESC LIMIT 10""")
        print("Fetching last 10 rows from lights table")
        lights = c.fetchall()
        for i in lights:
                room,lightvalue,entry = i
                line = {"light_value":lightvalue,"entry":entry}
                lights_list.append(line)
        return lights_list
    except Error as e:
        print (e)

    return None

### Temperture and humidity table ###
def create_temp(conn, temperature):
    """ Create a new temperature and humidity row into the temperature table """
    try:
        sql = """ INSERT INTO temperature(room,temperature,humidity,entry) VALUES(?,?,?,?) """
        c = conn.cursor()
        c.execute(sql, temperature)
        conn.commit()
        return int(c.lastrowid)
    except Error as e:
        print (e)

    return None

def select_last_temp(conn):
    """ Query last row in the temperature table """
    try:
        c = conn.cursor()
        c.execute("""SELECT * FROM temperature ORDER BY rowid DESC LIMIT 1""")
        print("Fetching latest temperature")
        temperature = c.fetchone()
    except Error as e:
        print (e)

    return temperature

def select_all_temp(conn):
    """ Query last 20 rows in the temp table """
    try:
        c = conn.cursor()
        c.execute(""" SELECT rowid, * FROM temperature ORDER BY rowid DESC LIMIT 20""")
        print("Fetching last 20 rows from temperature table")
        temperature = c.fetchall()
        return temperature
    except Error as e:
        print (e)

    return None

def delete_all_temps(conn):
    """ Delete all rows in the temperature table """
    try:
        sql = """ DELETE FROM temperature """
        cur = conn.cursor()
        cur.execute(sql)
        conn.commit()
    except Error as e:
        print (e)

    return None

### RFID table ###
def create_rfid(conn, rfid):
    """ Create a new rfid into the rfid table """
    try:
        sql = """ INSERT INTO rfid(room,uid,entry,vaild) VALUES(?,?,?,?) """
        c = conn.cursor()
        c.execute(sql, rfid)
        conn.commit()
        return int(c.lastrowid)
    except Error as e:
        print (e)

    return None

def select_all_rfid(conn):
    """ Query last 10 rfids in the rfid table """
    try:
        c = conn.cursor()
        c.execute(""" SELECT rowid, * FROM rfid ORDER BY rowid DESC LIMIT 10""")
        print("Fetching last 10 rows from rfid table")
        rfid = c.fetchall()
        return rfid
    except Error as e:
        print (e)

    return None

def delete_all_rfids(conn):
    """ Delete all rows in the rfid table """
    try:
        sql = """ DELETE FROM rfid """
        cur = conn.cursor()
        cur.execute(sql)
        conn.commit()
    except Error as e:
        print (e)

    return None

### RFID-USER table ###
def create_user(conn, user):
    """ Create a new user into the users table """
    try:
        sql = """ INSERT INTO users(user,rfid_uid) VALUES(?,?) """
        c = conn.cursor()
        c.execute(sql, user)
        conn.commit()
        return int(c.lastrowid)
    except Error as e:
        print (e)

    return None

def select_all_user(conn):
    """ Query all users in the users table """
    try:
        c = conn.cursor()
        c.execute(""" SELECT rowid, * FROM users""")
        print("Fetching all rows from users table")
        users = c.fetchall()
        return users
    except Error as e:
        print (e)

    return None

def delete_all_users(conn):
    """ Delete all rows in the users table """
    try:
        sql = """ DELETE FROM users """
        cur = conn.cursor()
        cur.execute(sql)
        conn.commit()
    except Error as e:
        print (e)

    return None
