from flask import redirect, render_template, request, session, url_for, Blueprint

room_status = Blueprint("room_status", __name__)
# Import SDK packages
from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTClient
import time
import sqlite3
import threading
import json
import os
from datetime import datetime
from threading import Thread
from app.models import *


host = "a26f9edjarcvo6.iot.us-west-2.amazonaws.com"
rootCAPath = "VeriSign-Class 3-Public-Primary-Certification-Authority-G5.pem"
certificatePath = "abc3cb84c2-certificate.pem.crt"
privateKeyPath = "abc3cb84c2-private.pem.key"

my_rpi = AWSIoTMQTTClient("serverunit")
my_rpi.configureEndpoint(host, 8883)
my_rpi.configureCredentials(rootCAPath, privateKeyPath, certificatePath)

my_rpi.configureOfflinePublishQueueing(-1)  # Infinite offline Publish queueing
my_rpi.configureDrainingFrequency(2)  # Draining: 2 Hz
my_rpi.configureConnectDisconnectTimeout(60)  # 10 sec
my_rpi.configureMQTTOperationTimeout(60)  # 5 sec


lights_conn = create_conn("app/lights.db")
sql_create_lights_table = """ CREATE TABLE if not exists lights(room INTEGER, lightvalue REAL,entry TIMESTAMP) """
lights_conn = create_table(lights_conn, sql_create_lights_table)


temps_conn = create_conn("app/temperature.db")
sql_create_temps_table = """ CREATE TABLE if not exists temperature(room INTEGER, temperature REAL, humidity REAL, entry TIMESTAMP)  """
temps_conn = create_table(temps_conn, sql_create_temps_table)


users_conn = create_conn("app/users.db")
sql_create_users_table = """ CREATE TABLE if not exists users(user TEXT, rfid_uid TEXT)  """
users_conn = create_table(users_conn, sql_create_users_table)
# user = ("shibao","[136, 4, 33, 188, 17]")
# create_user(users_conn, user)
# user = ("gerald","[136, 4, 154, 15, 25]")
# create_user(users_conn, user)
rfids_conn = create_conn("app/rfid.db")
sql_create_rfids_table = """ CREATE TABLE if not exists rfid(room INTEGER, uid TEXT, entry TIMESTAMP, vaild TEXT)  """
rfids_conn = create_table(rfids_conn, sql_create_rfids_table)


# Custom MQTT message callback
def customCallback(client, userdata, message):
    print("Received a new message: ")
    print(message.payload)
    jsonresponse = json.loads(message.payload)
    if jsonresponse["topic"] == "temps":
        tem = (jsonresponse["room"],jsonresponse["temp"],jsonresponse["humi"],datetime.now())
        create_temp(temps_conn, tem)
        print("INSERTED.")
    else:
        uid = (jsonresponse["uid"])
        room = (jsonresponse["room"])
        valid = check_user_id(users_conn, uid)
        if valid:
            user,fuid = valid
        else:
            user = "invalid".encode("utf-8")
            fuid = "invalid".encode("utf-8")
        user = user.encode("utf-8")
        room = room.encode("utf-8")
        addrfidentry = (room,fuid,datetime.now(),user)
        create_rfid(rfids_conn,addrfidentry)
        reply = {"user": user,"room":room}
        reply = str(reply)
        reply = reply.replace("\'","\"")
        reply = reply.replace("[", "\"[")
        reply = reply.replace("]", "]\"")

        my_rpi.publish("rfid/verification", str(reply),0)
    print("--------------\n\n")

def subscribetopic():

    # Connect and subscribe to AWS IoT
    print("Connecting to aws...")
    my_rpi.connect()

    topic = "lab/sensors"
    print("Starting subscription to.. {}".format(topic))

    print("Listening..")
    while True:
        my_rpi.subscribe(topic, 1, customCallback)
        time.sleep(1)
        print("Waiting.")


@room_status.route("/room_status", methods=["GET","POST"])
def room_status_main():
    data = get_data_homepage(temps_conn)
    room1temp,room1hum,room2temp,room2hum =data
    return render_template("room_status.html",title="Home",room1temp= room1temp,room1hum=room1hum,room2temp=room2temp,room2hum=room2hum)

@room_status.route("/")
def easteregg():
    return render_template("homepage.html")

t = threading.Thread(name='child procs', target=subscribetopic)
t.start()
